#!/bin/sh
##############################################################
# glurestore-raid.sh
#
# USAGE:
# Boot system with a rescue CD or any bootable medium
# that supports your type of disks (SATA/SCSI or IDE).
#
# Mount the medium containing the backed-up files and
# run the restore script:
# e.g. mount /dev/hda /mnt  
#      /mnt/glurestore-pclinuxos-raid.sh
#
# In this example the system is on /dev/sda and /dev/sdb
# Two partitions (2 and 3) are in RAID1 (mirror) configuration
# There are four primary partitions:
# /dev/sda1 and /dev/sdb1 will hold /boot (non-RAID)
# /dev/sda2 and /dev/sdb2 will hold / (/dev/md2)
# /dev/sda3 and /dev/sdb3 will hold /home (/dev/md3)
# /dev/sda4 and /dev/sdb4 are free (non-RAID)
#
# Boot (GRUB) information is written on the 
# Master Boot Records (MBR) of both disks
#
# 061116 jp/dm
# 061223 dm -- EOF correction
# 070503 sg -- root user and program availability checks
# 071220 dm -- remove hard-coded MAC address files (PCLinuxOS)
# 091217 dm -- raid (md) preparation
# 120207 dm - reread partitions, remove system-specific information
##############################################################
# Check that user is root
USER=`id -un`
if (test $USER != "root"); then
        echo "Error: This script must be executed by root"
        exit
fi

# Check availability of programs
TAR=`which tar 2>/dev/null`
if (test -z "$TAR"); then
        echo "Error: The \"tar\" program was not found"
        echo "       The live CD you are using does not contain the program \"tar\"."
        echo "       The backup procedure cannot be executed without it."
        echo "       Please use another live CD."
        exit
fi

# The sfdisk program may be available on your boot medium and/or
# your boot disk.  In the latter case, make sure that the sfdisk 
# program is compatible with the boot system.
#
SFDISK=`which sfdisk 2>/dev/null`
#SFDISK=`pwd`\/sfdisk  2>/dev/null

if (test -z "$SFDISK"); then
        echo "Error: The \"sfdisk\" program was not found"
        echo "       The live CD you are using does not contain the program \"sfdisk\"."
        echo "       The backup procedure cannot be executed without it."
        echo "       Please use another live CD."
        exit
fi

# The mdadm program may be available on your boot medium and/or
# your boot disk.  In the latter case, make sure that the mdadm 
# program is compatible with the boot system.
#
#MDADM=`which mdadm 2>/dev/null`
MDADM=`pwd`\/mdadm
#echo $MDADM
if (test -z "$MDADM"); then
        echo "Error: The \"mdadm\" program was not found"
        echo "       The live CD you are using does not contain the program \"mdadm\"."
        echo "       The backup procedure cannot be executed without it."
        echo "       Please use another live CD."
        exit
fi
GRUB=`which grub 2>/dev/null`
if (test -z "$GRUB"); then
        echo "Error: The \"grub\" program was not found"
        echo "       The live CD you are using does not contain the program \"grub\"."
        echo "       The backup procedure cannot be executed without it."
        echo "       Please use another live CD."
        exit
fi

MKEXT2=`which mkfs.ext2 2>/dev/null`
MKEXT3=`which mkfs.ext3 2>/dev/null`

if [[ -z "$MKEXT2" || -z "$MKEXT3" ]]; then
        echo "Error: Program \"mkfs.ext2\" or \"mkfs.ext3\" was not found"
        echo "       The live CD you are using does not contain this program."
        echo "       The backup procedure cannot be executed without it."
        echo "       Please use another live CD."
        exit
fi

# Checkpoint
#exit

THIS_DIR=`pwd`

# Optionally install Master Boot Record
# (uncomment the following two lines)
#echo "Installing MBR"
#dd if=mbr.bin of=/dev/sda bs=512 count=1

# Create Partitions
# If both disks have the same partitions
# there is no need for a separate sfdisk file.
echo "Creating Partition Tables"
"$SFDISK" -O "$THIS_DIR/disk_a_info" -f /dev/sda < sfdisk.sda
"$SFDISK" -O "$THIS_DIR/disk_b_info" -f /dev/sdb < sfdisk.sdb

# Reread partitions (with a short delay)
echo "Rereading partition tables (rescan, sfdisk)"
echo 1 > /sys/block/sda/device/rescan
sleep 1
echo 1 > /sys/block/sdb/device/rescan
sleep 1
"$SFDISK" -R /dev/sda
sleep 1
"$SFDISK" -R /dev/sdb
sleep 1

# Create RAID volumes
echo "Creating RAID Volumes"
modprobe raid1
"$MDADM" -C /dev/md2 -f -R -l1 -n2 /dev/sda2 /dev/sdb2
"$MDADM" -C /dev/md3 -f -R -l1 -n2 /dev/sda3 /dev/sdb3

# on a single disk...
#"$MDADM" -C /dev/md2 -f -R -l1 -n2 /dev/sda2 missing
#"$MDADM" -C /dev/md2 -R -l1 -n2 /dev/sda2 missing
#"$MDADM" -C /dev/md3 -R -l1 -n2 /dev/sda3 missing

# Checkpoint
#exit

# Format partitions
# Change to reflect the number of partitions that need
# to be formatted and the type of formatting
echo "Formatting disk 1 boot partition"
"$MKEXT2" /dev/sda1
echo "Formatting disk 2 boot partition"
"$MKEXT2" /dev/sdb1
echo "Formatting md2"
"$MKEXT3" /dev/md2
echo "Formatting md3"
"$MKEXT3" /dev/md3
#echo "Formatting disk1 empty partition"
#"$MKEXT3" /dev/sda4
#echo "Formatting disk2 empty partition"
#"$MKEXT3" /dev/sdb4

# Create directory to mount partitions; start with root (/) directory
echo "Creating directory to mount partitions "
mkdir /mnt2
mount /dev/md2 /mnt2

# Create /proc /sys and /mnt directories
echo "Creating proc, sys and mnt directories "
mkdir /mnt2/proc
mkdir /mnt2/sys
mkdir /mnt2/mnt

# Create directory and mount "home" partition
echo "Creating  directory and mounting home partition "
mkdir /mnt2/home
mount /dev/md3 /mnt2/home

# Create directory and mount "boot" partition
echo "Creating  directory and mounting boot partition (first disk)"
mkdir /mnt2/boot
mount /dev/sda1 /mnt2/boot

# Extract files
echo "Extracting files (root, home, boot1)"
"$TAR" -C /mnt2 -xpvzf syscopy.tgz
"$TAR" -C /mnt2/home -xpvzf homecopy.tgz
"$TAR" -C /mnt2/boot -xpvzf bootcopy.tgz

# Mount second boot directory and copy the boot files over
echo "Mounting and copying boot partition (second disk)"
mount /dev/sdb1 /mnt2/boot
echo "Extracting files (boot2)"
"$TAR" -C /mnt2/boot -xpvzf bootcopy.tgz

#####################################
# Optional commands may be placed here
# to remove hard-coded information or 
# to add system-specific parameters
#####################################

rm -f /mnt2/etc/iftab
rm -f /mnt2/etc/udev/rules.d/70-persistent-net.rules
rm -f /mnt1/var/lib/dbus/machine-id
rm -f /mnt2/root/.bash_history
rm -f /mnt2/home/ats/.bash_history

# Now make sure that correct boot information
# is present in the hard disk's MBR
# grub is run in batch mode
# hd0,0 is hard disk 0 (first hard disk of any kind), first partition – for this example
"$GRUB" --batch <<EOF
root (hd0,0)
setup (hd0)
quit
EOF

"$GRUB" --batch <<EOF
root (hd1,0)
setup (hd1)
quit
EOF

# Notify user
echo glurestore: finished!
# Reboot, if desirable
#reboot
